# 5 Feb. 2019
# this code takes in the variables pertaining to the EU catch allocation proposal, the catch history as outlined
# in that proposal, information on the IOTC members and which countries are sids, dcs, etc.
# 

#-------------------------------------------------------------------------------------------------------
# 
make_EU_allocation <- function( EU_simspecs=EU_ss[31,], input_dat = EU_inputdat,   
                                IOTC_data=IOTC_data, SIDS=SIDS, new.entrants = NULL, Corr.Fac= NULL)  { 
#--------------------------------------------------------------------------------------------------  
 # Running the code once will return the intermedieate steps between the historical allocation and the final catch 
 #    allocation : and return these variables, 
 #  [1] "SPECIES_CODE"     "flag"             "p_contrib"        "baseline_HA"      "base_CAF1"       
 #  [6] "base_CAF2"        "base_CAF3"        "base_newEntrants" "base_CorrFac"     "CA_Initial"      
 #  [11] "CA_wt_1"          "CA_final"  
 #  
# Run in a loop and save the CA_final using the CA_EU_Proposal_5Feb2019_runsims.r script
#--------------------------------------------------------------------------------------------------------------  
  
  # initial overall allocations
  base_factor   <-   EU_simspecs[ ,"Hist_prop"]
  comp_factor   <-   EU_simspecs[ ,"Comp_prop"]   
  corr_factor   <-   EU_simspecs[ ,"Corr_prop" ]  
  new_ent_factor   <-   EU_simspecs[ ,"NE_prop" ]  
  CAF_1              <-   EU_simspecs[ ,"CompF1"] 
  CAF_2              <-   EU_simspecs[ ,"CompF2"] 
  CAF_3              <-   EU_simspecs[ ,"CompF3"] 
  
  
  if( base_factor+ comp_factor+ new_ent_factor+corr_factor !=1 ) stop ("Error: base_factor+ compfactor+ new_ent_factor+corr_factordo not = 1 ")
  
  
  outputdat <-  left_join(input_dat, iotc_cnts, by='flag')%>% 
    mutate(baseline_HA = base_factor *   p_contrib)   # okay so the baseline historical allocation is the basefactor (85%) *  
  # the percent contribution over the period 2000-2016
  
  #  outputdat %>% group_by(SPECIES_CODE) %>% summarize(Total_allo = sum( baseline_HA ) )
  #
  outputdat <-    outputdat    %>%     #left_join(., ldcsINiotc[,c('flag', 'EEZ_KM2')], by='flag')  %>%
    
    mutate(ldcs = as.numeric(ldcs)) %>% 
    group_by(SPECIES_CODE) %>% 
    mutate( base_CAF1 = comp_factor*CAF_1*  (ldcs* EEZ_KM2/sum(ldcs*EEZ_KM2, na.rm=TRUE) ) ) %>%
    mutate( base_CAF1 = replace_na(base_CAF1, 0))
  # take care to make sure that countries with no previous catch get no allocation (no new fisheries created by complementary  factors) should not have to - this is just the fisheries with history.
  # check
  # outputdat %>%       group_by(SPECIES_CODE)%>% summarize(Total_allo = sum( baseline_HA , base_CAF1, na.rm=TRUE ) )
  #  
  # add in the Complenatary allocation for SIDS
  #  Small Island Developing States (SIDS) : shall recieve 1/4 of the complementary allocation as a supplement to the baseline,
  #   distributed amonst SIDS in prop. to EEZ size but not if they are LDCS
  # the ldcs list has not SIDS (i checked)
  outputdat <-  outputdat %>%left_join(., SIDS[,c('flag', 'prop.SIDS')], by='flag') %>%  mutate(prop.SIDS = replace_na(prop.SIDS, 0)) %>%
    group_by(SPECIES_CODE)%>%
    mutate( base_CAF2 =     comp_factor *CAF_2  * prop.SIDS/sum(prop.SIDS, na.rm=TRUE)) %>%
    mutate( base_CAF2 = replace_na(base_CAF2, 0))
  
  # # just to check  
  # head(outputdat2)
  # outputdat2[1:4,6:13]
  # # check
  # outputdat  %>% group_by(SPECIES_CODE)%>% summarize(Total_allo = sum( baseline_HA , base_CAF1, base_CAF2 , na.rm=TRUE ))    
  
  
  #------------------------------------------------
  # Add the Complementary Allocation for Costal Developing States
  # #  Coastal Developing States (CDS): Note not LDCs or SIDS  : shall recieve 1/4 of of the complementary allocation as a supplement to the baseline,
  #   distributed amonst CDS in prop. to EEZ size
  #
  DevCS <- iotc_cnts %>% dplyr::filter( type== "DCS", ldcs==0 ) %>% mutate( prop.DCS = round(EEZ_KM2/sum(EEZ_KM2),7))
  
  outputdat <-  outputdat %>%left_join(., DevCS[,c('flag', 'prop.DCS')],by='flag')  %>%  mutate(prop.DCS = replace_na(prop.DCS, 0)) %>%
    mutate( base_CAF3 =   prop.DCS *  comp_factor * CAF_3  )  %>%
    mutate( base_CAF3 = replace_na(base_CAF2, 0))
  
  #outputdat %>% group_by(SPECIES_CODE)%>% summarize(Total_allo = sum( baseline_HA , base_CAF1, base_CAF2 , base_CAF3 ))    
  
  
  #    so the leftover allocation should then go to 
  #    new entrants allocation [1%] 
  #    adjusted by correction factors [6%]....but we are punting on this so the LEFT over 
  #   
  #---------------------------New ENTRANTS - ----------------------------------------------------------------------
  #   If there are no requests by new-entrants or the actual allocation to new entrants is less than the set-aside of [1]%, 
  #   the remaining amount shall be distributed proportionally amongst Members on the basis of their final allocation.
  
  # lets assume that there are no new entrnats - 
  # and that the Correction factors don't get applied, so the 'leftover TAC' gets split proportionally  among members based on their final allocation base_CAF3 at this point
  
  # new_ent_factor <- 0.01
  # corr_factor <- 0.06
  # 
  #    
  if(is.null(  new.entrants ) ) {
    outputdat <- outputdat <- outputdat %>% left_join(., IOTC_data[,c("flag", "CP")], by='flag') %>%
      group_by(SPECIES_CODE )%>%
      mutate( base_newEntrants = new_ent_factor* ( CP / sum(CP ) ) ) } else
      {    outputdat <-  outputdat %>% full_join(.,new.entrants)  %>%    # this should just be a d.f. of flag and species and NewEnt =1
        group_by(SPECIES_CODE )%>%
        mutate( base_newEntrants = new_ent_factor* ( NewEnt / sum(NewEnt) ) )}  # 
  
  #outputdat %>% group_by(SPECIES_CODE)%>% summarize(Total_allo = sum( baseline_HA , base_CAF1, base_CAF2 , base_CAF3 , base_newEntrants ))    
  
  if(is.null(  Corr.Fac ) ) {
    outputdat <- outputdat %>%    # left_join(., IOTC_data[,c("flag", "CP")], by='flag') %>%
      group_by(SPECIES_CODE )%>%
      mutate( base_CorrFac = corr_factor* ( CP/ sum(CP ) ) ) } else
      { }#   outputdat <-  outputdat %>% full_join(.,corr_factor)  %>%    # this should just be a d.f. of flag and species and NewEnt =1
  #  group_by(SPECIES_CODE )  %>%
  #  mutate( base_CorrFac = corr_factor* ( corr_factor*/ sum(corr_factor) ) }  # 
  #outputdat %>% group_by(SPECIES_CODE)%>% summarize(Total_allo = sum( baseline_HA , base_CAF1, base_CAF2 , base_CAF3 , base_newEntrants, base_CorrFac ))    
  
  #--------------------------------------------------------------------------------------------------------------
  # at this point all has been allocated, except that the non members need to be downweighted by 80 %, and that needs to be re- adjusted 
  # so make a species/member specific ratio.
  # para 18
  # Cooperating non-contracting Parties shall be eligible for a maximum of 80% of its TAC as per paragraphs 8 to 12. Downwards revision of the TAC to Cooperating non-Contracting Parties shall lead to a proportional adjustment amongst CPCs on the basis of their final allocation. ...
  #  My take on this is reduce the final TAC for , Liberia, Senegal, and spread that to the Members 
  
  #--------------------------------------------------   
  cncp_adj <- 0.80
  CNCPs <- c("LBR",  "SEN") # 
  CNCP_allo <- as.data.frame( cbind(flag=CNCPs, prop_eligible= rep(cncp_adj, length(CNCPs) ) ))
  CNCP_allo$prop_eligible <- as.numeric(as.character(CNCP_allo$prop_eligible) )
  
  outputdat   <-    outputdat %>% group_by(SPECIES_CODE)%>% 
    mutate(CA_Initial =   baseline_HA+ base_CAF1+ base_CAF2 + base_CAF3 + base_newEntrants+base_CorrFac  )    %>%
    left_join(., CNCP_allo,by='flag')%>% 
    mutate(prop_eligible = replace_na(prop_eligible, 1))  %>% 
    mutate(CA_wt_1  =  prop_eligible*CA_Initial ) 
  
  
  
  # account for the eligibility of 80% fr the CNCPs
  LO_CNCP_TAC<-  outputdat%>% group_by(SPECIES_CODE) %>% summarize( CA_bySpecies=sum(CA_wt_1)) %>%  
    mutate(ELIG.CA = 1-CA_bySpecies ) 
  
  
  outputdat <- outputdat %>% left_join(., LO_CNCP_TAC[,c("SPECIES_CODE", "ELIG.CA")] , by = "SPECIES_CODE")  %>%
    mutate(status_num = ifelse(status == "member", 1,0)) %>%
    group_by(SPECIES_CODE) %>%         
    mutate(CA_final  = CA_wt_1 +  ELIG.CA* (status_num/ sum(status_num, na.rm=TRUE)) )
  
  #  outputdat %>% group_by(SPECIES_CODE) %>% summarize(CA_final=sum(CA_final))
  #summary(outputdat)
  #glimpse(outputdat)
  
  EU_results <- outputdat %>% 
    dplyr::select(  SPECIES_CODE, flag, p_contrib, baseline_HA, base_CAF1, base_CAF2, base_CAF3, base_newEntrants, base_CorrFac, CA_Initial, CA_wt_1, CA_final)%>%
    data.frame
  # dim(EU_results)
  return(EU_results)
}  # end function



